"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDispatcher = void 0;
const createDispatcher = ({ channel, configuration, exchanges, }) => {
    return {
        dispatch: async (queueName, payload, options = {}) => {
            const { isDelayed = false, publishOptions } = configuration.queues[queueName];
            const { delay, ...otherPublishOptions } = options;
            const headers = {
                ...publishOptions === null || publishOptions === void 0 ? void 0 : publishOptions.headers,
                ...otherPublishOptions === null || otherPublishOptions === void 0 ? void 0 : otherPublishOptions.headers,
            };
            if (delay) {
                headers["x-delay"] = delay;
            }
            const { exchange } = isDelayed ? exchanges.delayed : exchanges.direct;
            return channel.publish(exchange, queueName, Buffer.from(JSON.stringify(payload)), {
                ...publishOptions,
                ...otherPublishOptions,
                headers,
            });
        },
    };
};
exports.createDispatcher = createDispatcher;
