"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHive = void 0;
const createDispatcher_1 = require("./createDispatcher");
const createWorker_1 = require("./createWorker");
const createHive = async (connectionOrConnectionPromise, configuration) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
    const connection = "then" in connectionOrConnectionPromise
        ? await connectionOrConnectionPromise
        : connectionOrConnectionPromise;
    const channel = await connection.createChannel();
    const consumerTags = [];
    // Assert exchanges
    const exchanges = {
        direct: await channel.assertExchange(((_b = (_a = configuration.exchanges) === null || _a === void 0 ? void 0 : _a.direct) === null || _b === void 0 ? void 0 : _b.name) || "amqp-hive-direct", "direct", {
            ...(_d = (_c = configuration.exchanges) === null || _c === void 0 ? void 0 : _c.direct) === null || _d === void 0 ? void 0 : _d.options,
        }),
        delayed: await channel.assertExchange(((_f = (_e = configuration.exchanges) === null || _e === void 0 ? void 0 : _e.delayed) === null || _f === void 0 ? void 0 : _f.name) || "amqp-hive-delayed", "x-delayed-message", {
            ...(_h = (_g = configuration.exchanges) === null || _g === void 0 ? void 0 : _g.delayed) === null || _h === void 0 ? void 0 : _h.options,
            arguments: {
                ...(_l = (_k = (_j = configuration.exchanges) === null || _j === void 0 ? void 0 : _j.delayed) === null || _k === void 0 ? void 0 : _k.options) === null || _l === void 0 ? void 0 : _l.arguments,
                "x-delayed-type": "direct",
            },
        }),
    };
    // Assert queues
    await Promise.all(Object.keys(configuration.queues).map(async (queueName) => {
        var _a;
        const { queue } = await channel.assertQueue(queueName, {
            ...(_a = configuration.queues[queueName]) === null || _a === void 0 ? void 0 : _a.options,
        });
        const { isDelayed } = configuration.queues[queueName];
        const { exchange } = isDelayed ? exchanges.delayed : exchanges.direct;
        await channel.bindQueue(queue, exchange, queue);
    }));
    return {
        configuration,
        channel,
        connection,
        createDispatcher: () => {
            return createDispatcher_1.createDispatcher({ channel, configuration, exchanges });
        },
        createWorker: async (queues, context) => {
            // We add a `onReady` callback onto the provided configurations so we can keep track of
            // each consumer and `cancel` each one when `destroy` is called.
            return createWorker_1.createWorker({
                channel,
                context,
                queueConfigurations: Object.keys(queues).reduce((acc, queueName) => {
                    const { onMessage, options: { consumeOptions } = {} } = queues[queueName];
                    acc[queueName] = {
                        consumeOptions,
                        onMessage,
                        onReady: (consumerTag) => consumerTags.push(consumerTag),
                    };
                    return acc;
                }, {}),
            });
        },
        destroy: async () => {
            await Promise.all(consumerTags.map((consumerTag) => {
                return channel.cancel(consumerTag);
            }));
            await channel.close();
        },
    };
};
exports.createHive = createHive;
