"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createWorker = void 0;
const errors_1 = require("../errors");
const createWorker = async ({ channel, queueConfigurations, context, }) => {
    const queues = {};
    await Promise.all(Object.keys(queueConfigurations).map(async (name) => {
        const { consumeOptions, onMessage, onReady } = queueConfigurations[name];
        try {
            const { consumerTag } = await channel.consume(name, async (message) => {
                // Note: message will be null if the consumer is cancelled by RabbitMQ (https://www.rabbitmq.com/consumer-cancel.html)
                if (message) {
                    try {
                        await onMessage(JSON.parse(message.content.toString()), {
                            ...context,
                            message,
                        });
                        channel.ack(message);
                    }
                    catch (error) {
                        channel.nack(message, false, false);
                    }
                }
            }, {
                ...consumeOptions,
            });
            onReady(consumerTag);
            queues[name] = { consumerTag };
        }
        catch (error) {
            throw new errors_1.QueueConsumerInitializationFailureError(name, error);
        }
    }));
    return {
        queues,
    };
};
exports.createWorker = createWorker;
